/**
 *
 * Copyright 2005 The Apache Software Foundation or its licensors, as applicable.
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the License for the
 * specific language governing permissions and limitations under the License.
 */
package org.apache.tuscany.core.config.processor;

import java.util.List;

import org.apache.tuscany.model.assembly.Scope;
import org.apache.tuscany.model.assembly.Service;

/**
 * Provides utility functions for {@link org.apache.tuscany.core.extension.config.ImplementationProcessor}s
 *
 * @version $$Rev$$ $$Date$$
 */
public class ProcessorHelper {

    private ProcessorHelper() {
    }

    /**
     * Returns the first <code>Service</code> from a collection matching the interface type
     */
    public static Service getService(Class<?> interfaceType, List<Service> services) {
        for (Service service : services) {
            Class serviceInterface = service.getServiceContract().getInterface();
            if (serviceInterface.equals(interfaceType)) {
                return service;
            }
        }
        return null;
    }

    /**
     * Returns the scope enum specified by the annotation
     */
    public static Scope getScope(org.osoa.sca.annotations.Scope annotation) {
        if ("MODULE".equalsIgnoreCase(annotation.value())) {
            return Scope.MODULE;
        } else if ("SESSION".equalsIgnoreCase(annotation.value())) {
            return Scope.SESSION;
        } else if ("REQUEST".equalsIgnoreCase(annotation.value())) {
            return Scope.REQUEST;
        } else {
            return Scope.INSTANCE;
        }
    }

}
