/**
 *
 * Copyright 2005 The Apache Software Foundation
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */
package org.apache.tuscany.core.loader.assembly;

import javax.xml.namespace.QName;

import org.apache.tuscany.core.loader.StAXElementLoader;
import org.apache.tuscany.core.loader.StAXLoaderRegistry;
import org.apache.tuscany.core.system.annotation.Autowire;
import org.apache.tuscany.core.system.assembly.SystemAssemblyFactory;
import org.apache.tuscany.model.assembly.AssemblyObject;
import org.osoa.sca.annotations.Destroy;
import org.osoa.sca.annotations.Init;

/**
 * @version $Rev$ $Date$
 */
public abstract class AbstractLoader<T extends AssemblyObject> implements StAXElementLoader<T> {
    protected SystemAssemblyFactory factory;
    protected StAXLoaderRegistry registry;

    @Autowire
    public void setFactory(SystemAssemblyFactory factory) {
        this.factory = factory;
    }

    @Autowire
    public void setRegistry(StAXLoaderRegistry registry) {
        this.registry = registry;
    }

    @Init(eager = true)
    public void start() {
        registry.registerLoader(getXMLType(), this);
    }

    @Destroy
    public void stop() {
        registry.unregisterLoader(getXMLType(), this);
    }

    /**
     * Returns the QName of the element that this implementation handles.
     * @return the QName of the element that this implementation handles
     */
    protected abstract QName getXMLType();
}
